@@ -4,7 +4,8 @@ use pyo3::types::{PyBytes, PyDict};
 use pythonize::{depythonize, pythonize};
 use std::str::FromStr;
 use std::sync::Mutex;
-use vl_convert_rs::converter::VlOpts;
+use vl_convert_rs::converter::{VgOpts, VlOpts};
+use vl_convert_rs::html::bundle_vega_snippet;
 use vl_convert_rs::module_loader::import_map::VlVersion;
 use vl_convert_rs::serde_json;
 use vl_convert_rs::text::register_font_directory as register_font_directory_rs;
@@ -27,7 +28,7 @@ lazy_static! {
 ///
 /// Args:
 ///     vl_spec (str | dict): Vega-Lite JSON specification string or dict
-///     vl_version (str | None): Vega-Lite library version string (e.g. 'v5.5')
+///     vl_version (str | None): Vega-Lite library version string (e.g. 'v5.15')
 ///         (default to latest)
 ///     config (dict | None): Chart configuration object to apply during conversion
 ///     theme (str | None): Named theme (e.g. "dark") to apply during conversion
@@ -62,6 +63,7 @@ fn vegalite_to_vega(
             config,
             theme,
             show_warnings: show_warnings.unwrap_or(false),
+            allowed_base_urls: None,
         },
     )) {
         Ok(vega_spec) => vega_spec,
@@ -81,19 +83,22 @@ fn vegalite_to_vega(
 ///
 /// Args:
 ///     vg_spec (str | dict): Vega JSON specification string or dict
-///
+///     allowed_base_urls (list of str): List of allowed base URLs for external
+///                                      data requests. Default allows any base URL
 /// Returns:
 ///     str: SVG image string
 #[pyfunction]
-#[pyo3(text_signature = "(vg_spec)")]
-fn vega_to_svg(vg_spec: PyObject) -> PyResult<String> {
+#[pyo3(text_signature = "(vg_spec, allowed_base_urls)")]
+fn vega_to_svg(vg_spec: PyObject, allowed_base_urls: Option<Vec<String>>) -> PyResult<String> {
     let vg_spec = parse_json_spec(vg_spec)?;
 
     let mut converter = VL_CONVERTER
         .lock()
         .expect("Failed to acquire lock on Vega-Lite converter");
 
-    let svg = match PYTHON_RUNTIME.block_on(converter.vega_to_svg(vg_spec)) {
+    let svg = match PYTHON_RUNTIME
+        .block_on(converter.vega_to_svg(vg_spec, VgOpts { allowed_base_urls }))
+    {
         Ok(vega_spec) => vega_spec,
         Err(err) => {
             return Err(PyValueError::new_err(format!(
@@ -105,26 +110,65 @@ fn vega_to_svg(vg_spec: PyObject) -> PyResult<String> {
     Ok(svg)
 }
 
+/// Convert a Vega spec to a Vega Scenegraph
+///
+/// Args:
+///     vg_spec (str | dict): Vega JSON specification string or dict
+///     allowed_base_urls (list of str): List of allowed base URLs for external
+///                                      data requests. Default allows any base URL
+/// Returns:
+///     dict: scenegraph
+#[pyfunction]
+#[pyo3(text_signature = "(vg_spec, allowed_base_urls)")]
+fn vega_to_scenegraph(
+    vg_spec: PyObject,
+    allowed_base_urls: Option<Vec<String>>,
+) -> PyResult<PyObject> {
+    let vg_spec = parse_json_spec(vg_spec)?;
+
+    let mut converter = VL_CONVERTER
+        .lock()
+        .expect("Failed to acquire lock on Vega-Lite converter");
+
+    let sg = match PYTHON_RUNTIME
+        .block_on(converter.vega_to_scenegraph(vg_spec, VgOpts { allowed_base_urls }))
+    {
+        Ok(vega_spec) => vega_spec,
+        Err(err) => {
+            return Err(PyValueError::new_err(format!(
+                "Vega to Scenegraph conversion failed:\n{}",
+                err
+            )))
+        }
+    };
+    Python::with_gil(|py| -> PyResult<PyObject> {
+        pythonize(py, &sg).map_err(|err| PyValueError::new_err(err.to_string()))
+    })
+}
+
 /// Convert a Vega-Lite spec to an SVG image string using a
 /// particular version of the Vega-Lite JavaScript library.
 ///
 /// Args:
 ///     vl_spec (str | dict): Vega-Lite JSON specification string or dict
-///     vl_version (str | None): Vega-Lite library version string (e.g. 'v5.5')
+///     vl_version (str | None): Vega-Lite library version string (e.g. 'v5.15')
 ///         (default to latest)
 ///     config (dict | None): Chart configuration object to apply during conversion
 ///     theme (str | None): Named theme (e.g. "dark") to apply during conversion
 ///     show_warnings (bool | None): Whether to print Vega-Lite compilation warnings (default false)
+///     allowed_base_urls (list of str): List of allowed base URLs for external
+///                                      data requests. Default allows any base URL
 /// Returns:
 ///     str: SVG image string
 #[pyfunction]
-#[pyo3(text_signature = "(vl_spec, vl_version, config, theme, show_warnings)")]
+#[pyo3(text_signature = "(vl_spec, vl_version, config, theme, show_warnings, allowed_base_urls)")]
 fn vegalite_to_svg(
     vl_spec: PyObject,
     vl_version: Option<&str>,
     config: Option<PyObject>,
     theme: Option<String>,
     show_warnings: Option<bool>,
+    allowed_base_urls: Option<Vec<String>>,
 ) -> PyResult<String> {
     let vl_spec = parse_json_spec(vl_spec)?;
     let config = config.and_then(|c| parse_json_spec(c).ok());
@@ -146,6 +190,7 @@ fn vegalite_to_svg(
             config,
             theme,
             show_warnings: show_warnings.unwrap_or(false),
+            allowed_base_urls,
         },
     )) {
         Ok(vega_spec) => vega_spec,
@@ -159,25 +204,96 @@ fn vegalite_to_svg(
     Ok(svg)
 }
 
+/// Convert a Vega-Lite spec to a Vega Scenegraph using a
+/// particular version of the Vega-Lite JavaScript library.
+///
+/// Args:
+///     vl_spec (str | dict): Vega-Lite JSON specification string or dict
+///     vl_version (str | None): Vega-Lite library version string (e.g. 'v5.15')
+///         (default to latest)
+///     config (dict | None): Chart configuration object to apply during conversion
+///     theme (str | None): Named theme (e.g. "dark") to apply during conversion
+///     show_warnings (bool | None): Whether to print Vega-Lite compilation warnings (default false)
+///     allowed_base_urls (list of str): List of allowed base URLs for external
+///                                      data requests. Default allows any base URL
+/// Returns:
+///     str: SVG image string
+#[pyfunction]
+#[pyo3(text_signature = "(vl_spec, vl_version, config, theme, show_warnings, allowed_base_urls)")]
+fn vegalite_to_scenegraph(
+    vl_spec: PyObject,
+    vl_version: Option<&str>,
+    config: Option<PyObject>,
+    theme: Option<String>,
+    show_warnings: Option<bool>,
+    allowed_base_urls: Option<Vec<String>>,
+) -> PyResult<PyObject> {
+    let vl_spec = parse_json_spec(vl_spec)?;
+    let config = config.and_then(|c| parse_json_spec(c).ok());
+
+    let vl_version = if let Some(vl_version) = vl_version {
+        VlVersion::from_str(vl_version)?
+    } else {
+        Default::default()
+    };
+
+    let mut converter = VL_CONVERTER
+        .lock()
+        .expect("Failed to acquire lock on Vega-Lite converter");
+
+    let sg = match PYTHON_RUNTIME.block_on(converter.vegalite_to_scenegraph(
+        vl_spec,
+        VlOpts {
+            vl_version,
+            config,
+            theme,
+            show_warnings: show_warnings.unwrap_or(false),
+            allowed_base_urls,
+        },
+    )) {
+        Ok(vega_spec) => vega_spec,
+        Err(err) => {
+            return Err(PyValueError::new_err(format!(
+                "Vega-Lite to SVG conversion failed:\n{}",
+                err
+            )))
+        }
+    };
+    Python::with_gil(|py| -> PyResult<PyObject> {
+        pythonize(py, &sg).map_err(|err| PyValueError::new_err(err.to_string()))
+    })
+}
+
 /// Convert a Vega spec to PNG image data.
 ///
 /// Args:
 ///     vg_spec (str | dict): Vega JSON specification string or dict
 ///     scale (float): Image scale factor (default 1.0)
 ///     ppi (float): Pixels per inch (default 72)
-///
+///     allowed_base_urls (list of str): List of allowed base URLs for external
+///                                      data requests. Default allows any base URL
 /// Returns:
 ///     bytes: PNG image data
 #[pyfunction]
-#[pyo3(text_signature = "(vg_spec, scale, ppi)")]
-fn vega_to_png(vg_spec: PyObject, scale: Option<f32>, ppi: Option<f32>) -> PyResult<PyObject> {
+#[pyo3(text_signature = "(vg_spec, scale, ppi, allowed_base_urls)")]
+fn vega_to_png(
+    vg_spec: PyObject,
+    scale: Option<f32>,
+    ppi: Option<f32>,
+    allowed_base_urls: Option<Vec<String>>,
+) -> PyResult<PyObject> {
     let vg_spec = parse_json_spec(vg_spec)?;
 
     let mut converter = VL_CONVERTER
         .lock()
         .expect("Failed to acquire lock on Vega-Lite converter");
 
-    let png_data = match PYTHON_RUNTIME.block_on(converter.vega_to_png(vg_spec, scale, ppi)) {
+    let png_data = match PYTHON_RUNTIME.block_on(converter.vega_to_png(
+        vg_spec,
+        VgOpts { allowed_base_urls },
+        scale,
+        ppi,
+    )) {
         Ok(vega_spec) => vega_spec,
         Err(err) => {
             return Err(PyValueError::new_err(format!(
@@ -197,17 +313,22 @@ fn vega_to_png(vg_spec: PyObject, scale: Option<f32>, ppi: Option<f32>) -> PyRes
 ///
 /// Args:
 ///     vl_spec (str | dict): Vega-Lite JSON specification string or dict
-///     vl_version (str): Vega-Lite library version string (e.g. 'v5.5')
+///     vl_version (str): Vega-Lite library version string (e.g. 'v5.15')
 ///         (default to latest)
 ///     scale (float): Image scale factor (default 1.0)
 ///     ppi (float): Pixels per inch (default 72)
 ///     config (dict | None): Chart configuration object to apply during conversion
 ///     theme (str | None): Named theme (e.g. "dark") to apply during conversion
 ///     show_warnings (bool | None): Whether to print Vega-Lite compilation warnings (default false)
+///     allowed_base_urls (list of str): List of allowed base URLs for external
+///                                      data requests. Default allows any base URL
 /// Returns:
 ///     bytes: PNG image data
 #[pyfunction]
-#[pyo3(text_signature = "(vl_spec, vl_version, scale, ppi, config, theme, show_warnings)")]
+#[pyo3(
+    text_signature = "(vl_spec, vl_version, scale, ppi, config, theme, show_warnings, allowed_base_urls)"
+)]
+#[allow(clippy::too_many_arguments)]
 fn vegalite_to_png(
     vl_spec: PyObject,
     vl_version: Option<&str>,
@@ -216,6 +337,7 @@ fn vegalite_to_png(
     config: Option<PyObject>,
     theme: Option<String>,
     show_warnings: Option<bool>,
+    allowed_base_urls: Option<Vec<String>>,
 ) -> PyResult<PyObject> {
     let vl_version = if let Some(vl_version) = vl_version {
         VlVersion::from_str(vl_version)?
@@ -236,6 +358,7 @@ fn vegalite_to_png(
             config,
             theme,
             show_warnings: show_warnings.unwrap_or(false),
+            allowed_base_urls,
         },
         scale,
         ppi,
@@ -260,19 +383,30 @@ fn vegalite_to_png(
 ///     vg_spec (str | dict): Vega JSON specification string or dict
 ///     scale (float): Image scale factor (default 1.0)
 ///     quality (int): JPEG Quality between 0 (worst) and 100 (best). Default 90
-///
+///     allowed_base_urls (list of str): List of allowed base URLs for external
+///                                      data requests. Default allows any base URL
 /// Returns:
 ///     bytes: JPEG image data
 #[pyfunction]
-#[pyo3(text_signature = "(vg_spec, scale, quality)")]
-fn vega_to_jpeg(vg_spec: PyObject, scale: Option<f32>, quality: Option<u8>) -> PyResult<PyObject> {
+#[pyo3(text_signature = "(vg_spec, scale, quality, allowed_base_urls)")]
+fn vega_to_jpeg(
+    vg_spec: PyObject,
+    scale: Option<f32>,
+    quality: Option<u8>,
+    allowed_base_urls: Option<Vec<String>>,
+) -> PyResult<PyObject> {
     let vg_spec = parse_json_spec(vg_spec)?;
 
     let mut converter = VL_CONVERTER
         .lock()
         .expect("Failed to acquire lock on Vega-Lite converter");
 
-    let jpeg_data = match PYTHON_RUNTIME.block_on(converter.vega_to_jpeg(vg_spec, scale, quality)) {
+    let jpeg_data = match PYTHON_RUNTIME.block_on(converter.vega_to_jpeg(
+        vg_spec,
+        VgOpts { allowed_base_urls },
+        scale,
+        quality,
+    )) {
         Ok(vega_spec) => vega_spec,
         Err(err) => {
             return Err(PyValueError::new_err(format!(
@@ -292,17 +426,22 @@ fn vega_to_jpeg(vg_spec: PyObject, scale: Option<f32>, quality: Option<u8>) -> P
 ///
 /// Args:
 ///     vl_spec (str | dict): Vega-Lite JSON specification string or dict
-///     vl_version (str): Vega-Lite library version string (e.g. 'v5.5')
+///     vl_version (str): Vega-Lite library version string (e.g. 'v5.15')
 ///         (default to latest)
 ///     scale (float): Image scale factor (default 1.0)
 ///     quality (int): JPEG Quality between 0 (worst) and 100 (best). Default 90
 ///     config (dict | None): Chart configuration object to apply during conversion
 ///     theme (str | None): Named theme (e.g. "dark") to apply during conversion
 ///     show_warnings (bool | None): Whether to print Vega-Lite compilation warnings (default false)
+///     allowed_base_urls (list of str): List of allowed base URLs for external
+///                                      data requests. Default allows any base URL
 /// Returns:
 ///     bytes: JPEG image data
 #[pyfunction]
-#[pyo3(text_signature = "(vl_spec, vl_version, scale, quality, config, theme, show_warnings)")]
+#[pyo3(
+    text_signature = "(vl_spec, vl_version, scale, quality, config, theme, show_warnings, allowed_base_urls)"
+)]
+#[allow(clippy::too_many_arguments)]
 fn vegalite_to_jpeg(
     vl_spec: PyObject,
     vl_version: Option<&str>,
@@ -311,6 +450,7 @@ fn vegalite_to_jpeg(
     config: Option<PyObject>,
     theme: Option<String>,
     show_warnings: Option<bool>,
+    allowed_base_urls: Option<Vec<String>>,
 ) -> PyResult<PyObject> {
     let vl_version = if let Some(vl_version) = vl_version {
         VlVersion::from_str(vl_version)?
@@ -331,6 +471,7 @@ fn vegalite_to_jpeg(
             config,
             theme,
             show_warnings: show_warnings.unwrap_or(false),
+            allowed_base_urls,
         },
         scale,
         quality,
@@ -354,19 +495,28 @@ fn vegalite_to_jpeg(
 /// Args:
 ///     vg_spec (str | dict): Vega JSON specification string or dict
 ///     scale (float): Image scale factor (default 1.0)
-///
+///     allowed_base_urls (list of str): List of allowed base URLs for external
+///                                      data requests. Default allows any base URL
 /// Returns:
 ///     bytes: PDF file bytes
 #[pyfunction]
-#[pyo3(text_signature = "(vg_spec, scale)")]
-fn vega_to_pdf(vg_spec: PyObject, scale: Option<f32>) -> PyResult<PyObject> {
+#[pyo3(text_signature = "(vg_spec, scale, allowed_base_urls)")]
+fn vega_to_pdf(
+    vg_spec: PyObject,
+    scale: Option<f32>,
+    allowed_base_urls: Option<Vec<String>>,
+) -> PyResult<PyObject> {
     let vg_spec = parse_json_spec(vg_spec)?;
 
     let mut converter = VL_CONVERTER
         .lock()
         .expect("Failed to acquire lock on Vega-Lite converter");
 
-    let pdf_bytes = match PYTHON_RUNTIME.block_on(converter.vega_to_pdf(vg_spec, scale)) {
+    let pdf_bytes = match PYTHON_RUNTIME.block_on(converter.vega_to_pdf(
+        vg_spec,
+        VgOpts { allowed_base_urls },
+        scale,
+    )) {
         Ok(vega_spec) => vega_spec,
         Err(err) => {
             return Err(PyValueError::new_err(format!(
@@ -385,16 +535,17 @@ fn vega_to_pdf(vg_spec: PyObject, scale: Option<f32>) -> PyResult<PyObject> {
 ///
 /// Args:
 ///     vl_spec (str | dict): Vega-Lite JSON specification string or dict
-///     vl_version (str): Vega-Lite library version string (e.g. 'v5.5')
+///     vl_version (str): Vega-Lite library version string (e.g. 'v5.15')
 ///         (default to latest)
 ///     scale (float): Image scale factor (default 1.0)
 ///     config (dict | None): Chart configuration object to apply during conversion
 ///     theme (str | None): Named theme (e.g. "dark") to apply during conversion
-///
+///     allowed_base_urls (list of str): List of allowed base URLs for external
+///                                      data requests. Default allows any base URL
 /// Returns:
 ///     bytes: PDF image data
 #[pyfunction]
-#[pyo3(text_signature = "(vl_spec, vl_version, scale, config, theme)")]
+#[pyo3(text_signature = "(vl_spec, vl_version, scale, config, theme, allowed_base_urls)")]
 fn vegalite_to_pdf(
     vl_spec: PyObject,
     vl_version: Option<&str>,
@@ -402,6 +553,7 @@ fn vegalite_to_pdf(
     config: Option<PyObject>,
     theme: Option<String>,
     show_warnings: Option<bool>,
+    allowed_base_urls: Option<Vec<String>>,
 ) -> PyResult<PyObject> {
     let vl_version = if let Some(vl_version) = vl_version {
         VlVersion::from_str(vl_version)?
@@ -422,6 +574,7 @@ fn vegalite_to_pdf(
             config,
             theme,
             show_warnings: show_warnings.unwrap_or(false),
+            allowed_base_urls,
         },
         scale,
     )) {
@@ -473,6 +626,73 @@ fn vega_to_url(vg_spec: PyObject, fullscreen: Option<bool>) -> PyResult<String>
     )?)
 }
 
+/// Convert a Vega-Lite spec to self-contained HTML document using a particular
+/// version of the Vega-Lite JavaScript library.
+///
+/// Args:
+///     vl_spec (str | dict): Vega-Lite JSON specification string or dict
+///     vl_version (str): Vega-Lite library version string (e.g. 'v5.15')
+///         (default to latest)
+///     bundle (bool): If True, bundle all dependencies in HTML file
+///         If False (default), HTML file will load dependencies from only CDN
+///     config (dict | None): Chart configuration object to apply during conversion
+///     theme (str | None): Named theme (e.g. "dark") to apply during conversion
+///
+/// Returns:
+///     string: HTML document
+#[pyfunction]
+#[pyo3(text_signature = "(vl_spec, vl_version, bundle, config, theme)")]
+fn vegalite_to_html(
+    vl_spec: PyObject,
+    vl_version: Option<&str>,
+    bundle: Option<bool>,
+    config: Option<PyObject>,
+    theme: Option<String>,
+) -> PyResult<String> {
+    let vl_version = if let Some(vl_version) = vl_version {
+        VlVersion::from_str(vl_version)?
+    } else {
+        Default::default()
+    };
+    let vl_spec = parse_json_spec(vl_spec)?;
+    let config = config.and_then(|c| parse_json_spec(c).ok());
+
+    let mut converter = VL_CONVERTER
+        .lock()
+        .expect("Failed to acquire lock on Vega-Lite converter");
+
+    Ok(PYTHON_RUNTIME.block_on(converter.vegalite_to_html(
+        vl_spec,
+        VlOpts {
+            vl_version,
+            config,
+            theme,
+            show_warnings: false,
+            allowed_base_urls: None,
+        },
+        bundle.unwrap_or(false),
+    ))?)
+}
+
+/// Convert a Vega spec to a self-contained HTML document
+///
+/// Args:
+///     vg_spec (str | dict): Vega JSON specification string or dict
+///     bundle (bool): If True, bundle all dependencies in HTML file
+///         If False (default), HTML file will load dependencies from only CDN
+///
+/// Returns:
+///     string: HTML document
+#[pyfunction]
+#[pyo3(text_signature = "(vg_spec, bundle)")]
+fn vega_to_html(vg_spec: PyObject, bundle: Option<bool>) -> PyResult<String> {
+    let vg_spec = parse_json_spec(vg_spec)?;
+    let mut converter = VL_CONVERTER
+        .lock()
+        .expect("Failed to acquire lock on Vega-Lite converter");
+    Ok(PYTHON_RUNTIME.block_on(converter.vega_to_html(vg_spec, bundle.unwrap_or(false)))?)
+}
+
 /// Convert an SVG image string to PNG image data
 ///
 /// Args:
@@ -611,26 +831,68 @@ fn get_themes() -> PyResult<PyObject> {
     })
 }
 
+/// Create a JavaScript bundle containing the Vega Embed, Vega-Lite, and Vega libraries
+///
+/// Optionally, a JavaScript snippet may be provided that references Vega Embed
+/// as `vegaEmbed`, Vega-Lite as `vegaLite`, Vega and `vega`, and the lodash.debounce
+/// function as `lodashDebounce`.
+///
+/// The resulting string will include these JavaScript libraries and all of their dependencies.
+/// This bundle result is suitable for inclusion in an HTML <script> tag with no external
+/// dependencies required. The default snippet assigns `vegaEmbed`, `vegaLite`, and `vega`
+/// to the global window object, making them available globally to other script tags.
+///
+/// Args:
+///     snippet (str): An ES6 JavaScript snippet which includes no imports
+///     vl_version (str): Vega-Lite library version string (e.g. 'v5.15')
+///         (default to latest)
+/// Returns:
+///     str: Bundled snippet with all dependencies
+#[pyfunction]
+#[pyo3(text_signature = "(snippet, vl_version)")]
+fn javascript_bundle(snippet: Option<String>, vl_version: Option<&str>) -> PyResult<String> {
+    // Default to snippet that assigns them to the global window
+    let snippet = snippet.unwrap_or(
+        r#"
+        window.vegaEmbed=vegaEmbed;
+        window.vega=vega;
+        window.vegaLite=vegaLite;
+    "#
+        .to_string(),
+    );
+    let vl_version = if let Some(vl_version) = vl_version {
+        VlVersion::from_str(vl_version)?
+    } else {
+        Default::default()
+    };
+    Ok(PYTHON_RUNTIME.block_on(bundle_vega_snippet(&snippet, vl_version))?)
+}
+
 /// Convert Vega-Lite specifications to other formats
 #[pymodule]
 fn vl_convert(_py: Python, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(vegalite_to_vega, m)?)?;
     m.add_function(wrap_pyfunction!(vegalite_to_svg, m)?)?;
+    m.add_function(wrap_pyfunction!(vegalite_to_scenegraph, m)?)?;
     m.add_function(wrap_pyfunction!(vegalite_to_png, m)?)?;
     m.add_function(wrap_pyfunction!(vegalite_to_jpeg, m)?)?;
     m.add_function(wrap_pyfunction!(vegalite_to_pdf, m)?)?;
     m.add_function(wrap_pyfunction!(vegalite_to_url, m)?)?;
+    m.add_function(wrap_pyfunction!(vegalite_to_html, m)?)?;
     m.add_function(wrap_pyfunction!(vega_to_svg, m)?)?;
+    m.add_function(wrap_pyfunction!(vega_to_scenegraph, m)?)?;
     m.add_function(wrap_pyfunction!(vega_to_png, m)?)?;
     m.add_function(wrap_pyfunction!(vega_to_jpeg, m)?)?;
     m.add_function(wrap_pyfunction!(vega_to_pdf, m)?)?;
     m.add_function(wrap_pyfunction!(vega_to_url, m)?)?;
+    m.add_function(wrap_pyfunction!(vega_to_html, m)?)?;
     m.add_function(wrap_pyfunction!(svg_to_png, m)?)?;
     m.add_function(wrap_pyfunction!(svg_to_jpeg, m)?)?;
     m.add_function(wrap_pyfunction!(svg_to_pdf, m)?)?;
     m.add_function(wrap_pyfunction!(register_font_directory, m)?)?;
     m.add_function(wrap_pyfunction!(get_local_tz, m)?)?;
     m.add_function(wrap_pyfunction!(get_themes, m)?)?;
+    m.add_function(wrap_pyfunction!(javascript_bundle, m)?)?;
     m.add("__version__", env!("CARGO_PKG_VERSION"))?;
     Ok(())
 }
