@@ -1,6 +1,7 @@
 use crate::module_loader::import_map::{url_for_path, vega_themes_url, vega_url, VlVersion};
-use crate::module_loader::VlConvertModuleLoader;
+use crate::module_loader::{VlConvertModuleLoader, IMPORT_MAP};
 use std::borrow::Cow;
+use std::collections::hash_map::Entry;
 use std::collections::{HashMap, HashSet};
 use std::io::Cursor;
 use std::path::Path;
@@ -31,6 +32,7 @@ use png::{PixelDimensions, Unit};
 use tiny_skia::{Pixmap, PremultipliedColorU8};
 use usvg::{TreeParsing, TreeTextToPath};
 
+use crate::html::{bundle_vega_snippet, get_vega_script, get_vegalite_script};
 use image::io::Reader as ImageReader;
 
 use crate::text::{op_text_width, FONT_DB, USVG_OPTIONS};
@@ -45,12 +47,37 @@ lazy_static! {
     static ref NEXT_ARG_ID: Arc<Mutex<i32>> = Arc::new(Mutex::new(0));
 }
 
+#[derive(Debug, Clone, Default)]
+pub struct VgOpts {
+    pub allowed_base_urls: Option<Vec<String>>,
+}
+
 #[derive(Debug, Clone, Default)]
 pub struct VlOpts {
     pub config: Option<serde_json::Value>,
     pub theme: Option<String>,
     pub vl_version: VlVersion,
     pub show_warnings: bool,
+    pub allowed_base_urls: Option<Vec<String>>,
+}
+
+impl VlOpts {
+    pub fn to_embed_opts(&self) -> serde_json::Value {
+        let mut opts_map = serde_json::Map::new();
+
+        if let Some(theme) = &self.theme {
+            opts_map.insert(
+                "theme".to_string(),
+                serde_json::Value::String(theme.clone()),
+            );
+        }
+
+        if let Some(config) = &self.config {
+            opts_map.insert("config".to_string(), config.clone());
+        }
+
+        serde_json::Value::Object(opts_map)
+    }
 }
 
 fn set_json_arg(arg: serde_json::Value) -> Result<i32, AnyError> {
@@ -104,7 +131,6 @@ struct InnerVlConverter {
     worker: MainWorker,
     initialized_vl_versions: HashSet<VlVersion>,
     vega_initialized: bool,
-    module_loader: Rc<VlConvertModuleLoader>,
 }
 
 impl InnerVlConverter {
@@ -166,7 +192,7 @@ class WarningCollector {
             self.worker.run_event_loop(false).await?;
 
             // Override text width measurement in vega-scenegraph
-            for path in self.module_loader.import_map.keys() {
+            for path in IMPORT_MAP.keys() {
                 if path.ends_with("vega-scenegraph.js") {
                     let script_code = ModuleCode::from(format!(
                         r#"
@@ -198,16 +224,44 @@ import('{url}').then((sg) => {{
 
             // Create and initialize svg function string
             let function_str = r#"
-function vegaToSvg(vgSpec) {
+function vegaToView(vgSpec, allowedBaseUrls, errors) {
     let runtime = vega.parse(vgSpec);
-    const baseURL = 'https://vega.github.io/vega-datasets/';
+    let baseURL = 'https://vega.github.io/vega-datasets/';
     const loader = vega.loader({ mode: 'http', baseURL });
-    let view = new vega.View(runtime, {renderer: 'none', loader});
+    const originalHttp = loader.http.bind(loader);
+
+    if (allowedBaseUrls != null) {
+        loader.http = async (uri, options) => {
+            const parsedUri = new URL(uri);
+            if (
+                allowedBaseUrls.every(
+                    (allowedUrl) => !parsedUri.href.startsWith(allowedUrl),
+                )
+            ) {
+                errors.push(`External data url not allowed: ${uri}`);
+                throw new Error(`External data url not allowed: ${uri}`);
+            }
+            return originalHttp(uri, options);
+        };
+    }
+
+    return new vega.View(runtime, {renderer: 'none', loader});
+}
+
+function vegaToSvg(vgSpec, allowedBaseUrls, errors) {
+    let view = vegaToView(vgSpec, allowedBaseUrls, errors);
     let svgPromise = view.toSVG().finally(() => { view.finalize() });
     return svgPromise
 }
-"#;
 
+function vegaToScenegraph(vgSpec, allowedBaseUrls, errors) {
+    let view = vegaToView(vgSpec, allowedBaseUrls, errors);
+    let scenegraphPromise = view.runAsync().then(() => {
+        return JSON.parse(JSON.parse(vega.sceneToJSON(view.scenegraph())));
+    }).finally(() => { view.finalize() });
+    return scenegraphPromise
+}
+"#;
             self.worker
                 .execute_script("<anon>", deno_core::FastString::Static(function_str))?;
             self.worker.run_event_loop(false).await?;
@@ -258,24 +312,14 @@ function compileVegaLite_{ver_name}(vlSpec, config, theme, warnings) {{
     return {ver_name}.compile(vlSpec, options).spec
 }}
 
-function vegaLiteToSvg_{ver_name}(vlSpec, config, theme, warnings) {{
-    let options = {{}};
+function vegaLiteToSvg_{ver_name}(vlSpec, config, theme, warnings, allowedBaseUrls, errors) {{
+    let vgSpec = compileVegaLite_{ver_name}(vlSpec, config, theme, warnings);
+    return vegaToSvg(vgSpec, allowedBaseUrls, errors)
+}}
 
-    // Handle config and theme
-    let usermetaTheme = ((vlSpec.usermeta ?? {{}}).embedOptions ?? {{}}).theme;
-    let namedTheme = theme ?? usermetaTheme;
-    if (namedTheme != null) {{
-        options["config"] = vega.mergeConfig(vegaThemes[namedTheme], config ?? {{}});
-    }} else if (config != null) {{
-        options["config"] = config;
-    }}
-
-    if (!warnings) {{
-        options["logger"] = new WarningCollector();
-    }}
-
-    let vgSpec = {ver_name}.compile(vlSpec, options).spec;
-    return vegaToSvg(vgSpec)
+function vegaLiteToScenegraph_{ver_name}(vlSpec, config, theme, warnings, allowedBaseUrls, errors) {{
+    let vgSpec = compileVegaLite_{ver_name}(vlSpec, config, theme, warnings);
+    return vegaToScenegraph(vgSpec, allowedBaseUrls, errors)
 }}
 "#,
                 ver_name = format!("{:?}", vl_version),
@@ -292,7 +336,7 @@ function vegaLiteToSvg_{ver_name}(vlSpec, config, theme, warnings) {{
     }
 
     pub async fn try_new() -> Result<Self, AnyError> {
-        let module_loader = Rc::new(VlConvertModuleLoader::new());
+        let module_loader = Rc::new(VlConvertModuleLoader);
 
         let ext = Extension {
             name: "vl_convert_extensions",
@@ -333,6 +377,7 @@ function vegaLiteToSvg_{ver_name}(vlSpec, config, theme, warnings) {{
             stdio: Default::default(),
             should_wait_for_inspector_session: false,
             fs: Arc::new(RealFs),
+            feature_checker: Arc::new(Default::default()),
         };
 
         let main_module =
@@ -348,7 +393,6 @@ function vegaLiteToSvg_{ver_name}(vlSpec, config, theme, warnings) {{
             worker,
             initialized_vl_versions: Default::default(),
             vega_initialized: false,
-            module_loader,
         };
 
         Ok(this)
@@ -418,6 +462,9 @@ function vegaLiteToSvg_{ver_name}(vlSpec, config, theme, warnings) {{
             Some(s) => format!("'{}'", s),
         };
 
+        let allowed_base_urls =
+            serde_json::to_string(&serde_json::Value::from(vl_opts.allowed_base_urls))?;
+
         let code = format!(
             r#"
 compileVegaLite_{ver_name:?}(
@@ -425,6 +472,7 @@ compileVegaLite_{ver_name:?}(
     JSON.parse(Deno[Deno.internal].core.ops.op_get_json_arg({config_arg_id})),
     {theme_arg},
     {show_warnings},
+    {allowed_base_urls},
 )
 "#,
             ver_name = vl_opts.vl_version,
@@ -454,15 +502,24 @@ compileVegaLite_{ver_name:?}(
             Some(s) => format!("'{}'", s),
         };
 
+        let allowed_base_urls =
+            serde_json::to_string(&serde_json::Value::from(vl_opts.allowed_base_urls))?;
+
         let code = ModuleCode::from(format!(
             r#"
 var svg;
+var errors = [];
 vegaLiteToSvg_{ver_name:?}(
     JSON.parse(Deno[Deno.internal].core.ops.op_get_json_arg({spec_arg_id})),
     JSON.parse(Deno[Deno.internal].core.ops.op_get_json_arg({config_arg_id})),
     {theme_arg},
-    {show_warnings}
+    {show_warnings},
+    {allowed_base_urls},
+    errors,
 ).then((result) => {{
+    if (errors != null && errors.length > 0) {{
+        throw new Error(`${{errors}}`);
+    }}
     svg = result;
 }});
 "#,
@@ -479,20 +536,77 @@ vegaLiteToSvg_{ver_name:?}(
         Ok(value)
     }
 
-    pub async fn vega_to_svg(&mut self, vg_spec: &serde_json::Value) -> Result<String, AnyError> {
+    pub async fn vegalite_to_scenegraph(
+        &mut self,
+        vl_spec: &serde_json::Value,
+        vl_opts: VlOpts,
+    ) -> Result<serde_json::Value, AnyError> {
         self.init_vega().await?;
+        self.init_vl_version(&vl_opts.vl_version).await?;
+
+        let config = vl_opts.config.clone().unwrap_or(serde_json::Value::Null);
+        let spec_arg_id = set_json_arg(vl_spec.clone())?;
+        let config_arg_id = set_json_arg(config)?;
+        let theme_arg = match &vl_opts.theme {
+            None => "null".to_string(),
+            Some(s) => format!("'{}'", s),
+        };
+
+        let code = ModuleCode::from(format!(
+            r#"
+var sg;
+var errors = [];
+vegaLiteToScenegraph_{ver_name:?}(
+    JSON.parse(Deno[Deno.internal].core.ops.op_get_json_arg({spec_arg_id})),
+    JSON.parse(Deno[Deno.internal].core.ops.op_get_json_arg({config_arg_id})),
+    {theme_arg},
+    {show_warnings},
+    errors,
+).then((result) => {{
+    if (errors != null && errors.length > 0) {{
+        throw new Error(`${{errors}}`);
+    }}
+    sg = result;
+}})
+"#,
+            ver_name = vl_opts.vl_version,
+            spec_arg_id = spec_arg_id,
+            config_arg_id = config_arg_id,
+            theme_arg = theme_arg,
+            show_warnings = vl_opts.show_warnings,
+        ));
+        self.worker.execute_script("<anon>", code)?;
+        self.worker.run_event_loop(false).await?;
+
+        let value = self.execute_script_to_json("sg").await?;
+        Ok(value)
+    }
+
+    pub async fn vega_to_svg(
+        &mut self,
+        vg_spec: &serde_json::Value,
+        vg_opts: VgOpts,
+    ) -> Result<String, AnyError> {
+        self.init_vega().await?;
+        let allowed_base_urls =
+            serde_json::to_string(&serde_json::Value::from(vg_opts.allowed_base_urls))?;
 
         let arg_id = set_json_arg(vg_spec.clone())?;
         let code = ModuleCode::from(format!(
             r#"
 var svg;
+var errors = [];
 vegaToSvg(
-    JSON.parse(Deno[Deno.internal].core.ops.op_get_json_arg({arg_id}))
+    JSON.parse(Deno[Deno.internal].core.ops.op_get_json_arg({arg_id})),
+    {allowed_base_urls},
+    errors,
 ).then((result) => {{
+    if (errors != null && errors.length > 0) {{
+        throw new Error(`${{errors}}`);
+    }}
     svg = result;
 }})
-"#,
-            arg_id = arg_id
+"#
         ));
         self.worker.execute_script("<anon>", code)?;
         self.worker.run_event_loop(false).await?;
@@ -501,6 +615,34 @@ vegaToSvg(
         Ok(value)
     }
 
+    pub async fn vega_to_scenegraph(
+        &mut self,
+        vg_spec: &serde_json::Value,
+        vg_opts: VgOpts,
+    ) -> Result<serde_json::Value, AnyError> {
+        self.init_vega().await?;
+        let allowed_base_urls =
+            serde_json::to_string(&serde_json::Value::from(vg_opts.allowed_base_urls))?;
+
+        let arg_id = set_json_arg(vg_spec.clone())?;
+        let code = ModuleCode::from(format!(
+            r#"
+var sg;
+vegaToScenegraph(
+    JSON.parse(Deno[Deno.internal].core.ops.op_get_json_arg({arg_id})),
+    {allowed_base_urls},
+).then((result) => {{
+    sg = result;
+}})
+"#
+        ));
+        self.worker.execute_script("<anon>", code)?;
+        self.worker.run_event_loop(false).await?;
+
+        let value = self.execute_script_to_json("sg").await?;
+        Ok(value)
+    }
+
     pub async fn get_local_tz(&mut self) -> Result<Option<String>, AnyError> {
         let code = ModuleCode::from(
             "var localTz = Intl.DateTimeFormat().resolvedOptions().timeZone ?? 'undefined';"
@@ -544,13 +686,24 @@ pub enum VlConvertCommand {
     },
     VgToSvg {
         vg_spec: serde_json::Value,
+        vg_opts: VgOpts,
         responder: oneshot::Sender<Result<String, AnyError>>,
     },
+    VgToSg {
+        vg_spec: serde_json::Value,
+        vg_opts: VgOpts,
+        responder: oneshot::Sender<Result<serde_json::Value, AnyError>>,
+    },
     VlToSvg {
         vl_spec: serde_json::Value,
         vl_opts: VlOpts,
         responder: oneshot::Sender<Result<String, AnyError>>,
     },
+    VlToSg {
+        vl_spec: serde_json::Value,
+        vl_opts: VlOpts,
+        responder: oneshot::Sender<Result<serde_json::Value, AnyError>>,
+    },
     GetLocalTz {
         responder: oneshot::Sender<Result<Option<String>, AnyError>>,
     },
@@ -597,6 +750,7 @@ pub enum VlConvertCommand {
 pub struct VlConverter {
     sender: Sender<VlConvertCommand>,
     _handle: Arc<JoinHandle<Result<(), AnyError>>>,
+    _vegaembed_bundles: HashMap<VlVersion, String>,
 }
 
 impl VlConverter {
@@ -619,10 +773,22 @@ impl VlConverter {
                             let vega_spec = inner.vegalite_to_vega(&vl_spec, vl_opts).await;
                             responder.send(vega_spec).ok();
                         }
-                        VlConvertCommand::VgToSvg { vg_spec, responder } => {
-                            let svg_result = inner.vega_to_svg(&vg_spec).await;
+                        VlConvertCommand::VgToSvg {
+                            vg_spec,
+                            vg_opts,
+                            responder,
+                        } => {
+                            let svg_result = inner.vega_to_svg(&vg_spec, vg_opts).await;
                             responder.send(svg_result).ok();
                         }
+                        VlConvertCommand::VgToSg {
+                            vg_spec,
+                            vg_opts,
+                            responder,
+                        } => {
+                            let sg_result = inner.vega_to_scenegraph(&vg_spec, vg_opts).await;
+                            responder.send(sg_result).ok();
+                        }
                         VlConvertCommand::VlToSvg {
                             vl_spec,
                             vl_opts,
@@ -631,6 +797,14 @@ impl VlConverter {
                             let svg_result = inner.vegalite_to_svg(&vl_spec, vl_opts).await;
                             responder.send(svg_result).ok();
                         }
+                        VlConvertCommand::VlToSg {
+                            vl_spec,
+                            vl_opts,
+                            responder,
+                        } => {
+                            let sg_result = inner.vegalite_to_scenegraph(&vl_spec, vl_opts).await;
+                            responder.send(sg_result).ok();
+                        }
                         VlConvertCommand::GetLocalTz { responder } => {
                             let local_tz = inner.get_local_tz().await;
                             responder.send(local_tz).ok();
@@ -650,6 +824,7 @@ impl VlConverter {
         Self {
             sender,
             _handle: handle,
+            _vegaembed_bundles: Default::default(),
         }
     }
 
@@ -682,10 +857,15 @@ impl VlConverter {
         }
     }
 
-    pub async fn vega_to_svg(&mut self, vg_spec: serde_json::Value) -> Result<String, AnyError> {
+    pub async fn vega_to_svg(
+        &mut self,
+        vg_spec: serde_json::Value,
+        vg_opts: VgOpts,
+    ) -> Result<String, AnyError> {
         let (resp_tx, resp_rx) = oneshot::channel::<Result<String, AnyError>>();
         let cmd = VlConvertCommand::VgToSvg {
             vg_spec,
+            vg_opts,
             responder: resp_tx,
         };
 
@@ -706,6 +886,38 @@ impl VlConverter {
         }
     }
 
+    pub async fn vega_to_scenegraph(
+        &mut self,
+        vg_spec: serde_json::Value,
+        vg_opts: VgOpts,
+    ) -> Result<serde_json::Value, AnyError> {
+        let (resp_tx, resp_rx) = oneshot::channel::<Result<serde_json::Value, AnyError>>();
+        let cmd = VlConvertCommand::VgToSg {
+            vg_spec,
+            vg_opts,
+            responder: resp_tx,
+        };
+
+        // Send request
+        match self.sender.send(cmd).await {
+            Ok(_) => {
+                // All good
+            }
+            Err(err) => {
+                bail!(
+                    "Failed to send Scenegraph conversion request: {}",
+                    err.to_string()
+                )
+            }
+        }
+
+        // Wait for result
+        match resp_rx.await {
+            Ok(svg_result) => svg_result,
+            Err(err) => bail!("Failed to retrieve conversion result: {}", err.to_string()),
+        }
+    }
+
     pub async fn vegalite_to_svg(
         &mut self,
         vl_spec: serde_json::Value,
@@ -735,14 +947,47 @@ impl VlConverter {
         }
     }
 
+    pub async fn vegalite_to_scenegraph(
+        &mut self,
+        vl_spec: serde_json::Value,
+        vl_opts: VlOpts,
+    ) -> Result<serde_json::Value, AnyError> {
+        let (resp_tx, resp_rx) = oneshot::channel::<Result<serde_json::Value, AnyError>>();
+        let cmd = VlConvertCommand::VlToSg {
+            vl_spec,
+            vl_opts,
+            responder: resp_tx,
+        };
+
+        // Send request
+        match self.sender.send(cmd).await {
+            Ok(_) => {
+                // All good
+            }
+            Err(err) => {
+                bail!(
+                    "Failed to send Scenegraph conversion request: {}",
+                    err.to_string()
+                )
+            }
+        }
+
+        // Wait for result
+        match resp_rx.await {
+            Ok(svg_result) => svg_result,
+            Err(err) => bail!("Failed to retrieve conversion result: {}", err.to_string()),
+        }
+    }
+
     pub async fn vega_to_png(
         &mut self,
         vg_spec: serde_json::Value,
+        vg_opts: VgOpts,
         scale: Option<f32>,
         ppi: Option<f32>,
     ) -> Result<Vec<u8>, AnyError> {
         let scale = scale.unwrap_or(1.0);
-        let svg = self.vega_to_svg(vg_spec).await?;
+        let svg = self.vega_to_svg(vg_spec, vg_opts).await?;
         svg_to_png(&svg, scale, ppi)
     }
 
@@ -761,11 +1006,12 @@ impl VlConverter {
     pub async fn vega_to_jpeg(
         &mut self,
         vg_spec: serde_json::Value,
+        vg_opts: VgOpts,
         scale: Option<f32>,
         quality: Option<u8>,
     ) -> Result<Vec<u8>, AnyError> {
         let scale = scale.unwrap_or(1.0);
-        let svg = self.vega_to_svg(vg_spec).await?;
+        let svg = self.vega_to_svg(vg_spec, vg_opts).await?;
         svg_to_jpeg(&svg, scale, quality)
     }
 
@@ -784,10 +1030,11 @@ impl VlConverter {
     pub async fn vega_to_pdf(
         &mut self,
         vg_spec: serde_json::Value,
+        vg_opts: VgOpts,
         scale: Option<f32>,
     ) -> Result<Vec<u8>, AnyError> {
         let scale = scale.unwrap_or(1.0);
-        let svg = self.vega_to_svg(vg_spec).await?;
+        let svg = self.vega_to_svg(vg_spec, vg_opts).await?;
         svg_to_pdf(&svg, scale)
     }
 
@@ -802,6 +1049,94 @@ impl VlConverter {
         svg_to_pdf(&svg, scale)
     }
 
+    async fn get_vegaembed_bundle(&mut self, vl_version: VlVersion) -> Result<String, AnyError> {
+        // TODO: return &str and avoid cloning
+        let bundle = match self._vegaembed_bundles.entry(vl_version) {
+            Entry::Occupied(occupied) => occupied.get().clone(),
+            Entry::Vacant(vacant) => {
+                let bundle =
+                    bundle_vega_snippet("window.vegaEmbed = vegaEmbed;", vl_version).await?;
+                vacant.insert(bundle.clone());
+                bundle
+            }
+        };
+
+        Ok(bundle)
+    }
+
+    async fn build_html(
+        &mut self,
+        code: &str,
+        vl_version: VlVersion,
+        bundle: bool,
+    ) -> Result<String, AnyError> {
+        let script_tags = if bundle {
+            format!(
+                r#"
+    <script type="text/javascript">{}</script>
+            "#,
+                self.get_vegaembed_bundle(vl_version).await?
+            )
+        } else {
+            format!(
+                r#"
+    <script src="https://cdn.jsdelivr.net/npm/vega@5"></script>
+    <script src="https://cdn.jsdelivr.net/npm/vega-lite@{vl_ver}"></script>
+    <script src="https://cdn.jsdelivr.net/npm/vega-embed@6"></script>
+            "#,
+                vl_ver = vl_version.to_semver()
+            )
+        };
+
+        Ok(format!(
+            r#"<!DOCTYPE html>
+<html>
+  <head>
+    <style>
+        vega-chart.vega-embed {{
+          width: 100%;
+          display: flex;
+        }}
+        vega-chart.vega-embed details,
+        vega-chart.vega-embed details summary {{
+          position: relative;
+        }}
+    </style>
+    <meta charset="UTF-8">
+    <title>Chart</title>
+{script_tags}
+  </head>
+  <body>
+    <div id="vega-chart"></div>
+    <script type="text/javascript">
+{code}
+    </script>
+  </body>
+</html>
+        "#
+        ))
+    }
+
+    pub async fn vegalite_to_html(
+        &mut self,
+        vl_spec: serde_json::Value,
+        vl_opts: VlOpts,
+        bundle: bool,
+    ) -> Result<String, AnyError> {
+        let vl_version = vl_opts.vl_version;
+        let code = get_vegalite_script(vl_spec, vl_opts.to_embed_opts())?;
+        self.build_html(&code, vl_version, bundle).await
+    }
+
+    pub async fn vega_to_html(
+        &mut self,
+        vg_spec: serde_json::Value,
+        bundle: bool,
+    ) -> Result<String, AnyError> {
+        let code = get_vega_script(vg_spec)?;
+        self.build_html(&code, Default::default(), bundle).await
+    }
+
     pub async fn get_local_tz(&mut self) -> Result<Option<String>, AnyError> {
         let (resp_tx, resp_rx) = oneshot::channel::<Result<Option<String>, AnyError>>();
         let cmd = VlConvertCommand::GetLocalTz { responder: resp_tx };
